/*
********************************************************************************
Test23K256

Show how to use the SRAM23K256 library.
 
http://volker.top.geek.nz/arduino/

Copyright 2011 Volker Kuhlmann.
http://volker.top.geek.nz/contact.html

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

History:
 v1.1  7, 8 Jan 2011  Created
********************************************************************************
*/


/*
 Headers and Libraries
*/

// SPI
#include <SPI.h>

// SRAM23K256
#include <sram23K256.h>


/*
 Hardware configuration

 Note: Pin numbers are Arduino pin numbers, not IC pin numbers.
*/

// Serial RS-232C speed.
#define RS232_speed (115200)

// Chip select pin of the SRAM.
// SS (pin 10) is the ATmega default, but any can be used.
// Warning: When using a different pin, pin 10 MUST be left set as an output!
#define SRAM_CS_pin 10


/* 
 Global variables 
*/

// Object through which the SRAM is accessed.
SRAM23K256 sram;


/******************************************************************************
 Functions
 ******************************************************************************/

void testSRAM(void)
{
  byte b;
  unsigned int num, err;
  
  Serial.print("Status register: 0x");
  b = sram.readstatus();
  Serial.println(b, HEX);
  
  Serial.print("Byte at address 10: 0x");
  b = sram.read(10);
  Serial.println(b, HEX);
  
  Serial.println("Writing 0xA5 to address 327");
  sram.write(327, 0xA5);
  Serial.print("Byte at address 327: 0x");
  b = sram.read(327);
  Serial.println(b, HEX);
 
  Serial.println("Clearing memory");
  sram.sequential();
  sram.write(0, 0); // clear first byte
  for (uint16_t i = 1; i < SRAM23K256_MEMSIZE; ++i)
    SPI.transfer(0); // clear remaining bytes
  
  Serial.print("Non-zero bytes: ");
  err = 0;
  for (num = 0; num < SRAM23K256_MEMSIZE; num++)
    if (sram.read(num) != 0)
      err++;
  Serial.println(err);
  
  // MUCH FASTER:
  Serial.print("Non-zero bytes, MUCH FASTER: ");
  for (num = 0; num < SRAM23K256_MEMSIZE; num++) {
    if (num == 0)
      b = sram.read(0);
    else
      b = SPI.transfer(0); // any byte can be transmitted
    if (b != 0)
      err++;
  }
  Serial.println(err);
  
  // Fast, less code to write:
  Serial.print("Non-zero bytes, fast and shorter: ");
  sram.readstart(0);
  for (num = 0; num < SRAM23K256_MEMSIZE; num++) {
    if (SPI.transfer(0) != 0)
      err++;
  }
  Serial.println(err);
  
  Serial.println("Fill memory with 0xA5");
  sram.sequential(); // this setting persists
  sram.writestart(0);
  for (num = 0; num < SRAM23K256_MEMSIZE; num++)
    SPI.transfer(0xA5);
  
  Serial.print("Bytes not 0xA5: ");
  sram.readstart(0);
  for (num = 0; num < SRAM23K256_MEMSIZE; num++) {
    if (SPI.transfer(0) != 0xA5)
      err++;
  }
  Serial.println(err);
  
  Serial.println("Clearing memory");
  sram.clear(); // but only works for writing 0...
  // Note this has also set .sequential()
  
  Serial.println("Page access - address counter wraps around at end of page.");
  Serial.print("Memory size ");
  Serial.print(SRAM23K256_MEMSIZE, DEC);
  Serial.print(", page size ");
  Serial.println(SRAM23K256_PAGESIZE, DEC);
  Serial.println("Writing 10 bytes starting at address 25");
  sram.page();
  sram.write(25, 1);
  for (num = 1; num < 10; ++num)
    SPI.transfer(1);
  
  Serial.println("Start of memory is now (observe wrap-around):");
  sram.sequential();
  sram.readstart(0);
  for (num = 0; num < 16 * 3; ++num) {
    if ((num & 0x0F) == 0) {
      Serial.println();
      Serial.print(num);
      Serial.print(' ');
    }
    Serial.print(' ');
    Serial.print(SPI.transfer(0), DEC);
  }
  Serial.println();
}


/*
 Arduino functions
*/

void setup(void)
{
  // Open serial port back to computer for debugging output.
  Serial.begin(RS232_speed);

  delay(500);
  
  Serial.println("Startup...");

  // Start SPI interface
  SPI.begin();
  
  // Set SPI up for SRAM
  sram.begin(SRAM_CS_pin);
  
  testSRAM();
  
  // Free SPI interface for use by another device.
  sram.end();
  
  // Use the SPI interface in any other way.
  //...
  
  // Use SPI for SRAM again.
  // No need to specify the pin again.
  sram.begin();
  
  // More SRAM accesses
  //...
  
  // Stop using SRAM.
  sram.end();
  
  Serial.println("Finished");
}

void loop(void)      
{
  SPI.end();
  digitalWrite(13, 1); // LED on
  delay(150);
  digitalWrite(13, 0); // LED off
  delay(850);
}

/******************************************************************************/

