/*
********************************************************************************
sram23K256.cpp

Driver for the Microchip 23K256 32kbyte SRAM.

http://volker.top.geek.nz/arduino/

Copyright 2011 Volker Kuhlmann.
http://volker.top.geek.nz/contact.html

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

History:
 v1.0  07 Jan 2011  Created
 v1.1  08 Jan 2011  Finished, tested, ready for release.
********************************************************************************
*/

/*
 Features
*/

//#define DEBUGDELAY(arg) arg
// Remove all the debug delays and go full tilt:
#define DEBUGDELAY(arg) 


/*
 Headers and Libraries
*/

// _SFR_MEM*, etc
#include <avr/io.h>

// B... macros
#include <binary.h>

// Arduino environment
#include <WProgram.h>

// SPI
#include <SPI.h>

#include "sram23K256.h"


/*
 Variables
*/


/*
 Public Methods
*/

void SRAM23K256::begin(uint8_t CSpin)
{
  _cs = CSpin;
  
  begin();
}

void SRAM23K256::begin(void)
{
  // Assumption: SPI is enabled at this point (SPI.begin() was called).
  
  // Backup SPI registers.
  _SPCR = SPCR;
  //_SPSR = SPSR;
  
  pinMode(_cs, OUTPUT);
  disable();
  
  // Setup SPI.
  SPI.setDataMode(SPI_MODE0); // only mode 0 works
  SPI.setBitOrder(MSBFIRST);
  SPI.setClockDivider(SPI_CLOCK_DIV2); // fastest; SRAM chip 20MHz max
  DEBUGDELAY( SPI.setClockDivider(SPI_CLOCK_DIV128); );
  
  enable();
}

void SRAM23K256::end(void)
{
  // Disable / deselect chip.
  disable();
  
  // Restore registers.
  SPCR = _SPCR;
  
  // Do not turn off SPI here by calling SPI.end(). More than one SPI device may
  // be connected.
}

// Activate the chip select line.
void SRAM23K256::enable(void)
{
  // Calling enable() is much less code than calling digitalWrite(), at
  // negligible run-time penalty.
  digitalWrite(_cs, LOW);

  DEBUGDELAY( delay(1); )
}

// De-activate the chip select line.
void SRAM23K256::disable(void)
{
  DEBUGDELAY( delay(1); )
  digitalWrite(_cs, HIGH);
}

// De-select the chip briefly. Needed before it'll accept a new command.
void SRAM23K256::reenable(void)
{
  disable();
  DEBUGDELAY( delay(1); )
  enable();
}

// Return content of status register.
uint8_t SRAM23K256::readstatus(void)
{
  reenable();
  
  // In:  1 cmd byte
  // Out: 1 data byte
  SPI.transfer(SRAM23K256_RDSR);
  return SPI.transfer(0);
}

// Write given value into status register.
void SRAM23K256::writestatus(uint8_t data)
{
  reenable();
  
  // In:  1 cmd byte
  //      1 data byte
  SPI.transfer(SRAM23K256_WRSR);
  SPI.transfer(data);
}

// Set sequential access mode
// (address auto-increment, wrap-around at end of memory).
void SRAM23K256::sequential(void)
{
  uint8_t status;
  
  status = readstatus();
  status &= ~SRAM23K256_SR_MODE;
  status |= SRAM23K256_SR_SEQUMODE;
  writestatus(status);
}

// Set page mode
// (address auto-increment, wrap-around at end of page boundary).
void SRAM23K256::page(void)
{
  uint8_t status;
  
  status = readstatus();
  status &= ~SRAM23K256_SR_MODE;
  status |= SRAM23K256_SR_PAGEMODE;
  writestatus(status);
}

// Set byte mode
// (subsequent transfers access the same byte).
void SRAM23K256::samebyte(void)
{
  uint8_t status;
  
  status = readstatus();
  status &= ~SRAM23K256_SR_MODE;
  status |= SRAM23K256_SR_BYTEMODE;
  writestatus(status);
}

// Read a byte from the given address.
// Subsequent SPI transfers (until chip is deselected) access bytes as per
// access mode (sequential, page, same byte).
uint8_t SRAM23K256::read(uint16_t address)
{
  reenable();
  
  // Out: 1 cmd byte
  //      2 addr bytes (MSB first)
  // In:  1 data byte
  readstart(address);
  return SPI.transfer(0);
  
  // User may read as many more bytes as desired. Wrap-around at end of memory.
}

// Write a byte to the given address.
// Subsequent SPI transfers (until chip is deselected) access bytes as per
// access mode (sequential, page, same byte).
void SRAM23K256::write(uint16_t address, uint8_t data)
{
  // Out:  1 cmd byte
  //       2 addr bytes (MSB first)
  //       1 data byte
  writestart(address);
  SPI.transfer(data);

  // User may write as many more bytes as desired. Wrap-around at end of memory.
}

// Set the chip up for reading from the given address, but don't read any data.
void SRAM23K256::readstart(uint16_t address)
{
  reenable();
  
  // Out: 1 cmd byte
  //      2 addr bytes (MSB first)
  SPI.transfer(SRAM23K256_READ);
  SPI.transfer(highByte(address));
  SPI.transfer(lowByte(address));
};

// Set the chip up for writing from the given address, but don't write any data.
void SRAM23K256::writestart(uint16_t address)
{
  reenable();
  
  // Out: 1 cmd byte
  //      2 addr bytes (MSB first)
  SPI.transfer(SRAM23K256_WRITE);
  SPI.transfer(highByte(address));
  SPI.transfer(lowByte(address));
};

// Set sequential mode and clear all memory.
void SRAM23K256::clear(void)
{
  uint16_t addr;
  
  sequential();
  
  writestart(0);
  addr = 0;
  do
    SPI.transfer(0);
  while (++addr != SRAM23K256_MEMSIZE); // also works for 64kbyte
}

// Enable the -HOLD input in the status register.
void SRAM23K256::hold_enable(void)
{
  uint8_t status;
  
  status = readstatus();
  status &= ~SRAM23K256_SR_HOLDDISABLE;
  writestatus(status);
};

// Disable the -HOLD input in the status register.
void SRAM23K256::hold_disable(void)
{
  uint8_t status;
  
  status = readstatus();
  status |= SRAM23K256_SR_HOLDDISABLE;
  writestatus(status);
};

/******************************************************************************/

